import 'package:flutter/material.dart';
import 'jalaali_js.dart';

class PersianDatePicker {
  TextEditingController controller;
  String datetime;
  String finishDatetime;
  String gregorianDatetime;
  String gregorianFinishDatetime;
  String outputFormat;
  String rangeSeparator;
  double datePickerHeight;
  Duration yearSelectionAnimationDuration;
  Curve yearSelectionAnimationCurve;
  Duration monthSelectionAnimationDuration;
  Curve monthSelectionAnimationCurve;
  Color yearSelectionBackgroundColor;
  TextStyle yearSelectionTextStyle;
  Color yearSelectionHighlightBackgroundColor;
  TextStyle yearSelectionHighlightTextStyle;
  Color monthSelectionBackgroundColor;
  TextStyle monthSelectionTextStyle;
  TextStyle monthSelectionHighlightTextStyle;
  Color monthSelectionHighlightBackgroundColor;
  Color weekCaptionsBackgroundColor;
  TextStyle weekCaptionsTextStyle;
  Color headerBackgroundColor;
  TextStyle headerTextStyle;
  Color daysBackgroundColor;
  Color daysFontColor;
  Color currentDayBackgroundColor;
  Color currentDayFontColor;
  Color selectedDayBackgroundColor;
  Color selectedDayFontColor;
  Color headerTodayBackgroundColor;
  Color disabledDayBackgroundColor;
  Color disabledDayFontColor;
  Text headerTodayText;
  Icon headerTodayIcon;
  Color daysBorderColor;
  Color selectedDayBorderColor;
  Color selectedDaysInnerBorderColor;
  Color todayBorderColor;
  double daysBorderWidth;
  bool rangeDatePicker;

  PersianDatePicker(
      {@required this.controller,
        this.datetime,
        this.gregorianDatetime,
        this.finishDatetime,
        this.gregorianFinishDatetime,
        this.outputFormat = 'YYYY/MM/DD',
        this.rangeSeparator = ' # ',
        this.datePickerHeight = 320,
        this.rangeDatePicker = false,
        this.yearSelectionAnimationDuration = const Duration(milliseconds: 400),
        this.yearSelectionAnimationCurve = Curves.elasticOut,
        this.monthSelectionAnimationDuration = const Duration(milliseconds: 400),
        this.monthSelectionAnimationCurve = Curves.elasticOut,
        this.yearSelectionBackgroundColor = Colors.white,
        this.yearSelectionTextStyle =
        const TextStyle(fontSize: 16, color: Colors.black),
        this.yearSelectionHighlightBackgroundColor = const Color(0xffFAF4E1),
        this.yearSelectionHighlightTextStyle =
        const TextStyle(fontSize: 16, color: Color(0xff734505)),
        this.monthSelectionBackgroundColor = Colors.white,
        this.monthSelectionTextStyle =
        const TextStyle(fontSize: 16, color: Colors.black),
        this.monthSelectionHighlightBackgroundColor = const Color(0xffFAF4E1),
        this.monthSelectionHighlightTextStyle =
        const TextStyle(fontSize: 16, color: Color(0xff734505)),
        this.weekCaptionsBackgroundColor = Colors.blue,
        this.weekCaptionsTextStyle =
        const TextStyle(color: Colors.white, fontSize: 16),
        this.headerBackgroundColor = Colors.white,
        this.headerTextStyle = const TextStyle(color: Colors.black, fontSize: 18),
        this.daysBackgroundColor = Colors.transparent,
        this.daysFontColor = Colors.black87,
        this.currentDayBackgroundColor = const Color(0xffDDE8CA),
        this.currentDayFontColor = Colors.black,
        this.selectedDayBackgroundColor = const Color(0xffE9ECF5),
        this.selectedDayFontColor = Colors.black87,
        this.headerTodayBackgroundColor = const Color(0xffE0E8D3),
        this.disabledDayBackgroundColor = const Color(0xffEEEEEE),
        this.disabledDayFontColor = const Color(0xffA0A0A0),
        this.headerTodayText =
        const Text('امروز', style: TextStyle(fontSize: 19)),
        this.headerTodayIcon = const Icon(
          Icons.date_range,
          color: Colors.green,
        ),
        this.selectedDayBorderColor = const Color(0xffAEA8F0),
        this.selectedDaysInnerBorderColor = const Color(0xffD8D5F5),
        this.todayBorderColor = Colors.green,
        this.daysBorderColor =
        const Color(0xffEDEDED), //Colors.black.withOpacity(0.02),
        this.daysBorderWidth = 0.5});

  String _detectOutputSeparator(String outputFormat) {
    List<String> validCharacters = ['Y', 'M', 'D'];

    String separator = "/";

    outputFormat.runes.forEach((int rune) {
      var character = String.fromCharCode(rune);
      if (validCharacters.indexOf(character) < 0) {
        separator = character;
      }
    });

    return separator;
  }

  PersianDatePickerWidget initialize() {
    {
      String outputSeparator = _detectOutputSeparator(outputFormat);
      if (outputSeparator == rangeSeparator) {
        controller.text = 'جداکننده های محدوده و خروجی مشابه هستند';
        return null;
      }

      if ((datetime != null && finishDatetime != null) ||
          (gregorianDatetime != null && gregorianFinishDatetime != null))
        rangeDatePicker = true;

      String defaultInputFormat = 'YYYY/MM/DD';

      Jalaali jDatetime;
      Jalaali jFinishDatetime;
      Gregorian gregorianDate;
      Gregorian gregorianFinishDate;

      if (gregorianDatetime != null &&
          gregorianDatetime.toString().isNotEmpty) {
        List<String> gArray = gregorianDatetime.split("-");
        gregorianDate = Gregorian(
            int.parse(gArray[0]), int.parse(gArray[1]), int.parse(gArray[2]));

        if (gregorianFinishDatetime != null) {
          List<String> gfArray = gregorianFinishDatetime.split("-");
          gregorianFinishDate = Gregorian(int.parse(gfArray[0]),
              int.parse(gfArray[1]), int.parse(gfArray[2]));
        }

        if (gregorianFinishDate != null) {
          controller.text = translate(
              outputFormat, gregorianDate.toJalaali(), false) +
              rangeSeparator +
              translate(outputFormat, gregorianFinishDate.toJalaali(), false);
        } else {
          controller.text =
              translate(outputFormat, gregorianDate.toJalaali(), false);
        }
      } else {
//        jDatetime = Jalaali.now();

        if (datetime != null && datetime.toString().isNotEmpty) {
          // default format
          jDatetime = parseDatetimeInput(datetime, defaultInputFormat, "/");

          if (finishDatetime != null) {
            // default format
            jFinishDatetime =
                parseDatetimeInput(finishDatetime, defaultInputFormat, "/");
          }

          if (jFinishDatetime != null) {
            controller.text = translate(outputFormat, jDatetime, false) +
                rangeSeparator +
                translate(outputFormat, jFinishDatetime, false);
          } else {
            controller.text = translate(outputFormat, jDatetime, false);
          }
        }
      }

      Map<String, dynamic> options = {
        'separator': outputSeparator,
        'rangeSeparator': rangeSeparator,
        'outputFormat': outputFormat,
        'rangeDatePicker': rangeDatePicker,
        'controller': controller,
        'datePickerHeight': datePickerHeight,
        'yearSelectionAnimationDuration': yearSelectionAnimationDuration,
        'yearSelectionAnimationCurve': yearSelectionAnimationCurve,
        'yearSelectionHighlightBackgroundColor':
        yearSelectionHighlightBackgroundColor,
        'yearSelectionHighlightTextStyle': yearSelectionHighlightTextStyle,
        'monthSelectionAnimationDuration': monthSelectionAnimationDuration,
        'monthSelectionAnimationCurve': monthSelectionAnimationCurve,
        'monthSelectionHighlightBackgroundColor':
        monthSelectionHighlightBackgroundColor,
        'monthSelectionHighlightTextStyle': monthSelectionHighlightTextStyle,
        'yearSelectionBackgroundColor': yearSelectionBackgroundColor,
        'yearSelectionTextStyle': yearSelectionTextStyle,
        'monthSelectionBackgroundColor': monthSelectionBackgroundColor,
        'monthSelectionTextStyle': monthSelectionTextStyle,
        'weekCaptionsBackgroundColor': weekCaptionsBackgroundColor,
        'weekCaptionsTextStyle': weekCaptionsTextStyle,
        'headerBackgroundColor': headerBackgroundColor,
        'headerTextStyle': headerTextStyle,
        'daysBackgroundColor': daysBackgroundColor,
        'daysFontColor': daysFontColor,
        'currentDayBackgroundColor': currentDayBackgroundColor,
        'currentDayFontColor': currentDayFontColor,
        'selectedDayBackgroundColor': selectedDayBackgroundColor,
        'selectedDayFontColor': selectedDayFontColor,
        'headerTodayBackgroundColor': headerTodayBackgroundColor,
        'disabledDayBackgroundColor': disabledDayBackgroundColor,
        'disabledDayFontColor': disabledDayFontColor,
        'headerTodayText': headerTodayText,
        'headerTodayIcon': headerTodayIcon,
        'selectedDayBorderColor': selectedDayBorderColor,
        'selectedDaysInnerBorderColor': selectedDaysInnerBorderColor,
        'todayBorderColor': todayBorderColor,
        'daysBorderColor': daysBorderColor,
        'daysBorderWidth': daysBorderWidth
      };

      return PersianDatePickerWidget(options: options);
    }
  }
}

class PersianDatePickerWidget extends StatefulWidget {
  final Map<String, dynamic> options;

  PersianDatePickerWidget({
    this.options,
  });

  @override
  _PersianDatePickerWidgetState createState() =>
      _PersianDatePickerWidgetState();
}

parseDatetimeInput(datetime, format, separator) {
  List<String> inputParts = [];
  for (String item in datetime.split(separator)) {
    inputParts.add(item.trim());
  }

  List<String> formatParts = [];
  for (String item in format.split(separator)) {
    formatParts.add(item.trim());
  }

  List<String> dayFormats = ['DD', 'D'];
  List<String> monthFormats = ['MM', 'M'];
  List<String> yearFormats = ['YYYY', 'YY'];

  int day, month, year;

  for (int i = 0; i < formatParts.length; i++) {
    if (dayFormats.contains(formatParts[i])) {
      day = int.parse(inputParts[i]);
      continue;
    }
    if (monthFormats.contains(formatParts[i])) {
      month = int.parse(inputParts[i]);
      continue;
    }
    if (yearFormats.contains(formatParts[i])) {
      year = int.parse(inputParts[i]);
      continue;
    }
  }

  if (year != null && month != null && day != null) {
    return Jalaali(year, month, day);
  }

  return Jalaali.now();
}

class _PersianDatePickerWidgetState extends State<PersianDatePickerWidget>
    with TickerProviderStateMixin {
  final Jalaali currentDate = Jalaali.now();
  final Jalaali startDate =
  Jalaali.fromDateTime(DateTime(1925, 3, 21)); // 1320/1/1
  final List<Map<String, dynamic>> pages = [];
  final List<int> months = [];
  final List<int> years = [];

  Jalaali inputSelectedDate;
  Jalaali rangeStartDate;
  Jalaali rangeFinishDate;

  double datePickerHeight;

  int selectedPageIndex;
  int todayPageIndex;

  int datePickerCurrentMonth = 1;
  int datePickerCurrentYear = 1;

  bool inputSelectedDateByUser;

  ScrollController displayYearsController;
  ScrollController displayMonthsController;
  PageController datePickerController;

  AnimationController yearAnimationController;
  AnimationController monthAnimationController;

  inputStringToJalaali() {
    inputSelectedDate = Jalaali.now();
    inputSelectedDateByUser = false;

    if (widget.options['controller'].text != null &&
        widget.options['controller'].text.toString().isNotEmpty) {
      inputSelectedDateByUser = true;

      if (widget.options['rangeDatePicker'] == true) {
        List<String> datesList = widget.options['controller'].text
            .split(widget.options['rangeSeparator']);
        if (datesList.length == 2) {
          rangeStartDate = inputSelectedDate = parseDatetimeInput(datesList[0],
              widget.options['outputFormat'], widget.options['separator']);
          rangeFinishDate = parseDatetimeInput(datesList[1],
              widget.options['outputFormat'], widget.options['separator']);
        }
      } else {
        inputSelectedDate = parseDatetimeInput(
            widget.options['controller'].text,
            widget.options['outputFormat'],
            widget.options['separator']);
      }
    }
//
//    Jalaali datetime;
//    Jalaali finishDatetime;
//
//    if (widget.options['datetime !null = &&
//        widget.options['datetimtoStringe.().isNotEmpty) {
//
//      if(widget.options['rangeDatePicker =true= ){
//
//        List<String> startDate = widget.options['datetimsplite.(separator);
//        rangeStartDate = datetime = inputSelectedDate = Jalaali(
//            int.parse(startDate[0]), int.parse(startDate[1]), int.parse(startDate[2]));
//
//        List<String> finishDate = widget.options['finishDatetimsplite.(separator);
//        rangeFinishDate = finishDatetime = Jalaali(
//            int.parse(finishDate[0]), int.parse(finishDate[1]), int.parse(finishDate[2]));
//
//      } else {
//        List<String> sdl = widget.options['datetimtoStringe.().split(separator);
//        inputSelectedDate = datetime = Jalaali(
//            int.parse(sdl[0]), int.parse(sdl[1]), int.parse(sdl[2]));
//      }
//    } else {
//      inputSelectedDate = datetime = Jalaali.now();
//    }

//    if(finishDatetime != null) {
//      widget.options['controlletext r.= translate(widget.options['formatdatetime, , false) + _rangeSeparator + translate(widget.options['formatfinishDatetime, , false);
//    } else {
//      widget.options['controlletext r.= translate(widget.options['formatinputSelectedDate, , false);
//    }
  }

  @override
  void initState() {
    datePickerHeight = widget.options['datePickerHeight'];

    // change TextField input to a jalaali data-type, if there is no input, then current date is used
    inputStringToJalaali();

// set current month and year of the datepicker to selected input string
    datePickerCurrentMonth = inputSelectedDate.month;
    datePickerCurrentYear = inputSelectedDate.year;

    // using this index to determine the position of first day of month in week. for example ( may be 1st day of next month is monday )
    int firstDayOfMonthIndex = 0;

    // this variable is used to jump or animate to pages by using their index
    int pageIndex = 0;

    // to store number of days of previous month ( this can help to understand the firstDayOfMonthIndex )
    // initialize is 0 because we don't have any previous month
    int previousMonthDays = 0;

    // loop through years and months and add them inside pages ==>  List<Map<String, dynamic>>
    for (int year = startDate.year; year <= (currentDate.year + 100); year++) {
      // adding year to years list to have length of years
      years.add(year);

      for (int month = 1; month <= 12; month++) {
        // adding month to months list to have length of months
        months.add(month);

        // check if year and month of loops is equal to user input date. if yes, we store this page index as selected page index
        if (year == inputSelectedDate.year && month == inputSelectedDate.month)
          selectedPageIndex = pageIndex;

        // check if year and month of loops is equal to current datetime. if yes, we store this page index as today page index
        if (year == currentDate.year && month == currentDate.month)
          todayPageIndex = pageIndex;

        // by using jalaali package, calculate the number of days of current month
        int shamsiDaysOfMonth = Jalaali(year, month).monthLength;

        // by adding (shamsiDaysOfMonth + firstDayOfMonthInWeek) and % it to 7 ( number of days in a week )
        // we can find out what is the position of current month's first day inside the week. ( sat, sun, mon, ... )
        // at first, firstDayOfMonthInWeek = 0 so the first day of week is the first day of current month
        // firstDayOfMonthInWeek = 0 is important, because the starting date is 1925, 3, 21 which the first day of month is saturday
        // in farsi calendar, Saturday ( شنبه ) is the first day of week
        int firstDayOfMonthInWeek = firstDayOfMonthIndex % 7;

        // add current page to pages list
        pages.add({
          'Y': year,
          'M': month,
          'DIM': shamsiDaysOfMonth, // Days In Month
          'FDIW': firstDayOfMonthInWeek, // First Day of Month In Week
          'PMDIM': previousMonthDays // Previous Month Days In Month
        });

        pageIndex++;

        // we set current month's days as previousMonthDays for the next loop to store it in the next page
        previousMonthDays = shamsiDaysOfMonth;

        // calculating first day of month for the next loop round
        firstDayOfMonthIndex = (shamsiDaysOfMonth + firstDayOfMonthInWeek) % 7;
      }
    }

    // the whole datepicker is a PageView and so we define a controller for it
    datePickerController = PageController(initialPage: selectedPageIndex);

    // year animation controller
    yearAnimationController = AnimationController(
        vsync: this,
        duration: widget.options['yearSelectionAnimationDuration']);

    // month animation controller
    monthAnimationController = AnimationController(
        vsync: this,
        duration: widget.options['monthSelectionAnimationDuration']);

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
// in order to hide year and month little-dialogs when tapping anywhere on datepicker, we wrap our datepicker by a GestureDetector
    // and inside it's onTap reset both year and month animations
    return GestureDetector(
      onTap: () {
        yearAnimationController.reset();
        monthAnimationController.reset();
      },
      child: Directionality(
        textDirection: TextDirection.rtl,
        child: Container(
          height: datePickerHeight,
          child: Stack(
            children: <Widget>[
              Column(
                children: <Widget>[
                  Expanded(
                    child: Container(
                      child: PageView.builder(
                        onPageChanged: (index) {
                          // if page is changed, swiped, we change datepicker current month and year and chaning datepicker state
                          Map<String, dynamic> pageInfo = pages[index];
                          datePickerCurrentMonth = pageInfo['M'];
                          datePickerCurrentYear = pageInfo['Y'];
                          setState(() {});
                        },
                        controller: datePickerController,

                        // the number of pages
                        itemCount: pages.length,

                        // building pages
                        itemBuilder: (BuildContext context, int index) {
                          // getting page info from pages list by it's index
                          Map<String, dynamic> pageInfo = pages[index];

                          // store the parts of page inside a list of widgets
                          List<Widget> datePicker = [];

                          // in most cases the number of rows of days for each page is 7
                          // but in some cases, if the starting day of current month is ( thu, fri )
                          // the number of rows may be 8. so we check it here
                          int viewPortDays = 35;
                          if ((pageInfo['DIM'] + pageInfo['FDIW']) > 35) {
                            viewPortDays = 42;
                          }

                          // rendering days row by row.
                          // first row starts by 0 index
                          int previousRowDayNumber = 0;
                          for (int row = 1; row <= (viewPortDays / 7); row++) {
                            List<Widget> rowWidgets = [];

                            // if it is first row, then we need to display headers before days rows above current row of days,
                            // and also display previous month days before current month starting day
                            if (row == 1) {
                              datePicker.addAll([
                                _datePickerPageHeader(pageInfo),
                                _datePickerPageWeekCaptions(),
                              ]);

                              // displaying previous month days
                              for (int daysBefore = (pageInfo['PMDIM'] -
                                  pageInfo['FDIW'] +
                                  1);
                              daysBefore <= pageInfo['PMDIM'];
                              daysBefore++) {
                                rowWidgets.add(_dayBlock(
                                    day: daysBefore,
                                    otherMonthDay: true,
                                    pageInfo: pageInfo));
                              }

                              // displaying current month days in this row
                              for (int firstRowDays = 1;
                              firstRowDays <= (7 - pageInfo['FDIW']);
                              firstRowDays++) {
                                rowWidgets.add(_dayBlock(
                                    day: firstRowDays, pageInfo: pageInfo));
                              }

                              previousRowDayNumber = (7 - pageInfo['FDIW']);

// if this row is the last row
                            } else if (row == (viewPortDays / 7)) {
                              // display days of current row
                              for (int lastRowDays = previousRowDayNumber + 1;
                              lastRowDays <= pageInfo['DIM'];
                              lastRowDays++) {
                                rowWidgets.add(_dayBlock(
                                    day: lastRowDays, pageInfo: pageInfo));
                              }

                              // display days of next month right after the last day of current month
                              for (int daysAfter = viewPortDays;
                              daysAfter >
                                  (pageInfo['DIM'] + pageInfo['FDIW']);
                              daysAfter--) {
                                rowWidgets.add(_dayBlock(
                                    day: viewPortDays - daysAfter + 1,
                                    otherMonthDay: true,
                                    pageInfo: pageInfo));
                              }

                              // if this row is not first and last...
                            } else {
                              // displaying days of current row
                              for (int dayRow = previousRowDayNumber + 1;
                              dayRow <= (previousRowDayNumber + 7);
                              dayRow++) {
                                rowWidgets.add(
                                    _dayBlock(day: dayRow, pageInfo: pageInfo));
                              }

                              previousRowDayNumber += 7;
                            }

                            datePicker.add(Expanded(
                              flex: 1,
                              child: Row(
                                children: rowWidgets,
                              ),
                            ));
                          }

                          return Column(
                            mainAxisSize: MainAxisSize.max,
                            children: datePicker,
                          );
                        },
                      ),
                    ),
                  )
                ],
              ),

              // years dialog widget
              Positioned(
                top: 0,
                left: 0,
                right: 0,
                child: ScaleTransition(
                  scale: CurvedAnimation(
                      parent: yearAnimationController,
                      curve: Interval(0.0, 1.0,
                          curve:
                          widget.options['yearSelectionAnimationCurve'])),
                  child: Center(
                    child: Container(
                      height: 200,
                      width: 100,
                      decoration:
                      BoxDecoration(color: Colors.white, boxShadow: [
                        BoxShadow(
                          color: Colors.black54,
                          blurRadius: 5,
                          spreadRadius: 1,
                        )
                      ]),
                      child: displayYears(),
                    ),
                  ),
                ),
              ),

// month dialog widget
              Positioned(
                top: 0,
                left: 0,
                right: 0,
                child: ScaleTransition(
                  scale: CurvedAnimation(
                      parent: monthAnimationController,
                      curve: Interval(0.0, 1.0,
                          curve:
                          widget.options['monthSelectionAnimationCurve'])),
                  child: Center(
                    child: Container(
                      height: 200,
                      width: 100,
                      decoration:
                      BoxDecoration(color: Colors.white, boxShadow: [
                        BoxShadow(
                          color: Colors.black54,
                          blurRadius: 5,
                          spreadRadius: 1,
                        )
                      ]),
                      child: displayMonths(),
                    ),
                  ),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  // this function displays each day as a block or cell
  Widget _dayBlock({int day, bool otherMonthDay = false, pageInfo}) {
    Color color = widget.options['daysBackgroundColor'];
    Color fontColor = widget.options['daysFontColor'];
    Jalaali dayBlockDateTime =
    Jalaali(datePickerCurrentYear, datePickerCurrentMonth, day);
    Border dayBorder = Border.all(
        color: widget.options['daysBorderColor'],
        width: widget.options['daysBorderWidth']);

    // if datepicker type is range and starting day is selected, then all previous days should be disabled
    bool rangeModeTrueAndDayIsBeforeStart = false;
    if (rangeStartDate != null && rangeFinishDate == null) {
      if (dayBlockDateTime.toDateTime().isBefore(rangeStartDate.toDateTime())) {
        rangeModeTrueAndDayIsBeforeStart = true;
      }

      if (day == rangeStartDate.day &&
          datePickerCurrentMonth == rangeStartDate.month &&
          datePickerCurrentYear == rangeStartDate.year &&
          otherMonthDay == false) {
        BorderSide bs = BorderSide(
            color: widget.options['selectedDayBorderColor'],
            width: widget.options['daysBorderWidth']);
        dayBorder = Border(top: bs, right: bs, bottom: bs);
        color = widget.options['selectedDayBackgroundColor'];
      }
    }

    // if current day is not a day of current month ( it belongs to previous or next month ) then it should be disabled
    if (otherMonthDay == true || rangeModeTrueAndDayIsBeforeStart == true) {
      color = widget.options['disabledDayBackgroundColor'];
      fontColor = widget.options['disabledDayFontColor'];
    } else {
      // if current day is belongs to current month and equals to current date , then change color and font to current date
      if (currentDate.day == day &&
          currentDate.month == datePickerCurrentMonth &&
          currentDate.year == datePickerCurrentYear) {
        color = widget.options['currentDayBackgroundColor'];
        fontColor = widget.options['currentDayFontColor'];
        dayBorder = Border.all(
            color: widget.options['todayBorderColor'],
            width: widget.options['daysBorderWidth']);
      }

      // if current day is belongs to current month and equals to selected date , then change color and font to selected date
      if (day == inputSelectedDate.day &&
          datePickerCurrentMonth == inputSelectedDate.month &&
          datePickerCurrentYear == inputSelectedDate.year &&
          inputSelectedDateByUser == true) {
        color = widget.options['selectedDayBackgroundColor'];
        fontColor = widget.options['selectedDayFontColor'];

        dayBorder = Border.all(
            color: widget.options['selectedDayBorderColor'],
            width: widget.options['daysBorderWidth']);
      }

      // if current day is belongs to current month and is inside the selected range , then change color and font to selected date
      if ((rangeStartDate != null && rangeFinishDate != null)) {
        DateTime dayBlockGregorianDateTime = dayBlockDateTime.toDateTime();

        if (dayBlockGregorianDateTime.compareTo(rangeStartDate.toDateTime()) >=
            0 &&
            dayBlockGregorianDateTime.compareTo(rangeFinishDate.toDateTime()) <=
                0) {
          color = widget.options['selectedDayBackgroundColor'];
          fontColor = widget.options['selectedDayFontColor'];

          DateTime neighbourDay;
          BorderSide borderTop = BorderSide(
              color: widget.options['selectedDaysInnerBorderColor'],
              width: widget.options['daysBorderWidth']);
          BorderSide borderRight = BorderSide(
              color: widget.options['selectedDaysInnerBorderColor'],
              width: widget.options['daysBorderWidth']);
          BorderSide borderLeft = BorderSide(
              color: widget.options['selectedDaysInnerBorderColor'],
              width: widget.options['daysBorderWidth']);
          BorderSide borderBottom = BorderSide(
              color: widget.options['selectedDaysInnerBorderColor'],
              width: widget.options['daysBorderWidth']);
//          BorderSide borderTop = BorderSide(color: widget.options['daysBorderColor'], width: widget.options['daysBorderWidth']);
//          BorderSide borderTop = BorderSide(color: widget.options['daysBorderColor'], width: widget.options['daysBorderWidth']);

          /*Border Top*/
          neighbourDay = dayBlockGregorianDateTime.subtract(Duration(days: 7));
          if (neighbourDay.compareTo(rangeStartDate.toDateTime()) < 0 ||
              neighbourDay.compareTo(rangeFinishDate.toDateTime()) > 0) {
            borderTop = BorderSide(
                color: widget.options['selectedDayBorderColor'],
                width: widget.options['daysBorderWidth']);
          }

          /*Border Right*/
          neighbourDay = dayBlockGregorianDateTime.subtract(Duration(days: 1));
          if (neighbourDay.compareTo(rangeStartDate.toDateTime()) < 0 ||
              neighbourDay.compareTo(rangeFinishDate.toDateTime()) > 0) {
            borderRight = BorderSide(
                color: widget.options['selectedDayBorderColor'],
                width: widget.options['daysBorderWidth']);
          }

          /*Border Left*/
          neighbourDay = dayBlockGregorianDateTime.add(Duration(days: 1));
          if (neighbourDay.compareTo(rangeStartDate.toDateTime()) < 0 ||
              neighbourDay.compareTo(rangeFinishDate.toDateTime()) > 0) {
            borderLeft = BorderSide(
                color: widget.options['selectedDayBorderColor'],
                width: widget.options['daysBorderWidth']);
          }

          /*Border Bottom*/
          neighbourDay = dayBlockGregorianDateTime.add(Duration(days: 7));
          if (neighbourDay.compareTo(rangeStartDate.toDateTime()) < 0 ||
              neighbourDay.compareTo(rangeFinishDate.toDateTime()) > 0) {
            borderBottom = BorderSide(
                color: widget.options['selectedDayBorderColor'],
                width: widget.options['daysBorderWidth']);
          }

          dayBorder = Border(
              top: borderTop,
              right: borderRight,
              left: borderLeft,
              bottom: borderBottom);

//
//          print(dayBlockGregorianDateTime.toString() + ' => ' + dayBlockDateTime.toString());
//
//
//              gregorianOfDayBlock = Gregorian(dayBlockGregorianDateTime.year, dayBlockGregorianDateTime.month, dayBlockGregorianDateTime.day);
//
//          print(dayBlockGregorianDateTime.toString() + ' => ' + gregorianOfDayBlock.toJalaali().toString());

        }
      }
    }

    return Expanded(
      child: Material(
        child: InkWell(
          onTap: () {
            if (otherMonthDay == false &&
                rangeModeTrueAndDayIsBeforeStart == false) {
              // by tapping on any day, both year and month dialogs should disappear and their animations should reset
              yearAnimationController.reset();
              monthAnimationController.reset();

              if (widget.options['rangeDatePicker'] == true) {
                // if start date is selected, then waiting for finish date selection and clearing input to display new selection
                if (rangeStartDate == null ||
                    (rangeStartDate != null && rangeFinishDate != null)) {
                  rangeStartDate = Jalaali(pageInfo['Y'], pageInfo['M'], day);
                  rangeFinishDate = null;
                  widget.options['controller'].text = '';
                } else {
                  rangeFinishDate = Jalaali(pageInfo['Y'], pageInfo['M'], day);

                  if (rangeStartDate
                      .toDateTime()
                      .compareTo(rangeFinishDate.toDateTime()) <=
                      0) {
                    widget.options['controller'].text = translate(
                        widget.options['outputFormat'],
                        rangeStartDate,
                        false) +
                        widget.options['rangeSeparator'] +
                        translate(widget.options['outputFormat'],
                            rangeFinishDate, false);
                  } else {
                    widget.options['controller'].text = '';
                  }

                  // after selecting both start and finish,
                  // we update TextField text and then
                  // we set both start and finish to null so user can select another range
                  rangeStartDate = null;
                  rangeFinishDate = null;
                }
              } else {
                Jalaali jd = Jalaali(pageInfo['Y'], pageInfo['M'], day);
                widget.options['controller'].text =
                    translate(widget.options['outputFormat'], jd, false);
              }

              // after updating input by selected date, we convert the selected date to jalaali date
              inputStringToJalaali();

              setState(() {});
            }
          },
          child: Container(
//            padding: EdgeInsets.symmetric(vertical: 10, horizontal: 10),
            decoration: BoxDecoration(
              color: color,
              border: dayBorder,
            ),
            child: Center(
              child: Text(
                day.toString(),
                style: TextStyle(color: fontColor, fontSize: 16),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget _datePickerPageHeader(pageInfo) {
    Jalaali datetime = Jalaali(pageInfo['Y'], pageInfo['M'], 1);
    return Container(
        color: widget.options['headerBackgroundColor'],
        height: 50,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Expanded(
              flex: 3,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  GestureDetector(
                    onTap: () {
                      yearAnimationController.reset();
                      if (monthAnimationController.isDismissed)
                        monthAnimationController.forward();
                      else
                        monthAnimationController.reverse();
                    },
                    child: Container(
                      alignment: Alignment.center,
                      padding: EdgeInsets.symmetric(horizontal: 15),
                      child: Row(
                        children: <Widget>[
                          Container(
                            child: Icon(
                              Icons.expand_more,
                              size: widget.options['headerTextStyle'].fontSize,
                              color: widget.options['headerTextStyle'].color,
                            ),
                          ),
                          SizedBox(width: 4),
                          Container(
                            child: Text(
                              translate('MMM', datetime, false),
                              style: widget.options['headerTextStyle'],
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
                  SizedBox(width: 10),
                  GestureDetector(
                      onTap: () {
                        monthAnimationController.reset();
                        if (yearAnimationController.isDismissed)
                          yearAnimationController.forward();
                        else
                          yearAnimationController.reverse();
                      },
                      child: Container(
                        alignment: Alignment.center,
                        padding: EdgeInsets.symmetric(horizontal: 15),
                        child: Row(
                          children: <Widget>[
                            Container(
                              child: Text(datetime.year.toString(),
                                  style: widget.options['headerTextStyle']),
                            ),
                            SizedBox(width: 8),
                            Container(
                              margin: EdgeInsets.only(top: 1),
                              child: Icon(
                                Icons.expand_more,
                                size:
                                widget.options['headerTextStyle'].fontSize,
                                color: widget.options['headerTextStyle'].color,
                              ),
                            ),
                          ],
                        ),
                      ))
                ],
              ),
            ),
            Expanded(
              flex: 1,
              child: InkWell(
                onTap: () {
                  datePickerController.animateToPage(todayPageIndex,
                      duration: Duration(milliseconds: 500),
                      curve: Curves.easeIn);
                },
                child: Container(
                  alignment: Alignment.center,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      widget.options['headerTodayIcon'],
                      SizedBox(
                        width: 5,
                      ),
                      Container(
                          padding: EdgeInsets.only(top: 3),
                          child: widget.options['headerTodayText'])
                    ],
                  ),
                  color: widget.options['headerTodayBackgroundColor'],
                ),
              ),
            )
          ],
        ));
  }

  Widget _datePickerPageWeekCaptions() {
    return Row(
      children: <Widget>[
        _weekDayCaption('ش'),
        _weekDayCaption('ی'),
        _weekDayCaption('د'),
        _weekDayCaption('س'),
        _weekDayCaption('چ'),
        _weekDayCaption('پ'),
        _weekDayCaption('ج'),
      ],
    );
  }

  Widget _weekDayCaption(String title) {
    return Expanded(
      child: Container(
          padding: EdgeInsets.all(6),
          color: widget.options['weekCaptionsBackgroundColor'],
          child: Center(
              child: Text(
                title,
                style: widget.options['weekCaptionsTextStyle'],
              ))),
    );
  }

  Widget displayYears() {
    double itemHeight = 50;

    int selectedYearIndex = years.indexOf(inputSelectedDate.year);
    displayYearsController = ScrollController(
        initialScrollOffset: (selectedYearIndex * itemHeight).toDouble());

    return ListView.builder(
        scrollDirection: Axis.vertical,
        controller: displayYearsController,
        itemCount: years.length,
        itemBuilder: (BuildContext context, int index) {
          Color bgColor = widget.options['yearSelectionBackgroundColor'];
          TextStyle textStyle = widget.options['yearSelectionTextStyle'];
          if (years[index] == datePickerCurrentYear) {
            bgColor = widget.options['yearSelectionHighlightBackgroundColor'];
            textStyle = widget.options['yearSelectionHighlightTextStyle'];
          }

          return GestureDetector(
            onTap: () {
              datePickerCurrentYear = years[index];
              setState(() {});
              datePickerController.animateToPage(
                  (datePickerCurrentYear - startDate.year) * 12 +
                      datePickerCurrentMonth -
                      1,
                  duration: Duration(milliseconds: 400),
                  curve: Curves.easeOut);
              yearAnimationController.reset();
            },
            child: Container(
              height: itemHeight,
              padding: EdgeInsets.only(top: 2),
              decoration: BoxDecoration(
                  border: Border(top: BorderSide(color: Colors.black12)),
                  color: bgColor),
//            margin: EdgeInsets.symmetric(vertical: 2.0),
              child: Center(
                  child: Text(
                    years[index].toString(),
                    style: textStyle,
                  )),
            ),
          );
        });
  }

  Widget displayMonths() {
    double itemHeight = 50;

    displayMonthsController = ScrollController(
        initialScrollOffset: (inputSelectedDate.month * itemHeight).toDouble());

    return ListView.builder(
        scrollDirection: Axis.vertical,
        controller: displayMonthsController,
        itemCount: 12,
        itemBuilder: (BuildContext context, int index) {
          Jalaali datetime = Jalaali(inputSelectedDate.year, index + 1, 1);

          Color bgColor = widget.options['monthSelectionBackgroundColor'];
          TextStyle textStyle = widget.options['monthSelectionTextStyle'];
          if (index + 1 == datePickerCurrentMonth) {
            bgColor = widget.options['monthSelectionHighlightBackgroundColor'];
            textStyle = widget.options['monthSelectionHighlightTextStyle'];
          }

          return GestureDetector(
            onTap: () {
              datePickerCurrentMonth = index + 1;
              setState(() {});
              datePickerController.animateToPage(
                  (datePickerCurrentYear - startDate.year) * 12 + index,
                  duration: Duration(milliseconds: 400),
                  curve: Curves.easeOut);
              monthAnimationController.reset();
            },
            child: Container(
              height: itemHeight,
              padding: EdgeInsets.only(top: 2),
              decoration: BoxDecoration(
                  border: Border(top: BorderSide(color: Colors.black12)),
                  color: bgColor),
              child: Center(
                  child: Text(
                    translate('MMM', datetime, false),
                    style: textStyle,
                  )),
            ),
          );
        });
  }
}
